% Fig 8-30
% In the PIN receiver, ThermalNoiseDen should be reduced to 1e-24;% [A^2/Hz]
% ThermalNoiseDen (default) = 1e-22;% [A^2/Hz]

clear all ;% clear all variables
close all

%============== Define global variables ===================================================
global BitRate Bits TimeWindow SampleRate DataLength samplesperbit t_range f_sym Ref_Frequency

Ref_Frequency = 193.1e12; %[Hz] Reference optical frequency

BitRate = 10e9;  
Bits2Power = 10;
Bits = 2^Bits2Power;
TimeWindow = Bits/BitRate;
SampleRate = BitRate*2^10;
DataLength = TimeWindow*SampleRate;
samplesperbit = SampleRate/BitRate; % data number per bit

% time, optical frequency range
t_range = 0:1/SampleRate:1/SampleRate*(DataLength-1);
f_sym = SampleRate*(-DataLength/2:(DataLength/2-1))/DataLength; %

%==========================================================================================
%=================== Define Simulation Blocks =============================================
%------------- Transmitter -------------------%
% SM Laser
UnmodulatedPower = 2e-3; %[W] 
Linewidth = 10e6; %[Hz]
Laser1 = LaserCW(UnmodulatedPower,Linewidth);

PreSpaces = 2;  %number of leading zero bits
PostSpaces = 2; %number of trailing zero bits
info_bits = infobits_PRBS(PreSpaces,PostSpaces);

E_data = NRZCoder(info_bits);

ERiseTime = 1/4/BitRate; %  In the form of k/BitRate, k should be less than (1/2)*RZRatio if RZCoder used   
E_elec = RiseTimeAdjust(E_data,ERiseTime);

ModulationIndex = 0.9;
mod_out = ModulatorAmplitude(Laser1,E_elec,ModulationIndex);

%---------------- Transmission -------------------------------------
%=========== Fiber Parameters +++++++++++++++++========
S_Fiber_Length = 60e3; %[m]
S_FiberAttenuation =  20/S_Fiber_Length ; %[dB/m]
S_Dispersion = 16e-6;   %[s/m^2]
S_DispersionSlope = 0.08e3;   %[s/m^3]

%--------------------------------------------------------------------------
[SMF_out1 yf1] = SingleModeFiber_Lin(mod_out,S_Fiber_Length,S_FiberAttenuation,S_Dispersion,S_DispersionSlope);

%--------------- Receiver1 (PIN) ------------------------------------------
Responsivity = 1.0;% [A/W], [0,2]
%ThermalNoiseDen = 1e-22;% [A^2/Hz] 
ThermalNoiseDen = 1e-24;% [A^2/Hz] 

DarkCurrent = 0;% [A], [0,10e-6]
ShotNoiseFactor = 1;%   (1 if shot noise included, 0 if not)

y_PIN = PhotoDiode_PIN(SMF_out1,Responsivity,ThermalNoiseDen,DarkCurrent,ShotNoiseFactor); 

FilterOrder = 3;
BW3dB = 0.75*BitRate;

y_PIN_LP = E_FilterLPBessel(y_PIN,FilterOrder,BW3dB);
Out = EyeAnalyzer_Relative(y_PIN_LP,info_bits,0.5);

%--------------- Q & BER Calculation --------------------------------------

[BER_noISI,i_th,Q] = qnber_noISI(y_PIN_LP,info_bits)
[BER_ISI,i_th,Q] = qnber_ISI(y_PIN_LP,info_bits)


