clear all ;% clear all variables
close all

%============== Define global variables ===================================================
global BitRate Bits TimeWindow SampleRate DataLength samplesperbit t_range f_sym Ref_Frequency

BitRate = 1e9;  
Bits2Power = 5;
Bits = 2^Bits2Power;
TimeWindow = Bits/BitRate;
SampleRate = BitRate*2^10;
DataLength = TimeWindow*SampleRate;
Ref_Frequency = 193.1e12; %[Hz] Reference optical frequency
samplesperbit = SampleRate/BitRate; % data number per bit

% time, optical frequency range
t_range = 0:1/SampleRate:1/SampleRate*(DataLength-1);
f_sym = SampleRate*(-DataLength/2:(DataLength/2-1))/DataLength; %

%==========================================================================================
%=================== Define Simulation Blocks =============================================
% SM Laser
UnmodulatedPower = 2e-3; %[W] 
Linewidth = 10e6; %[Hz]
Laser1 = LaserCW(UnmodulatedPower,Linewidth);
%------------- Data Source -------------------%
PreSpaces = 2;  %number of leading zero bits
PostSpaces = 2; %number of trailing zero bits
info_bits = infobits_PRBS(PreSpaces,PostSpaces);

E_data = NRZCoder(info_bits);

ERiseTime = 1/4/BitRate; %  In the form of k/BitRate, k should be less than (1/2)*RZRatio if RZCoder used   
E_elec = RiseTimeAdjust(E_data,ERiseTime);

ER1 = 10;% Extinction Ratio [dB]
ModulationIndex1 = 1-10^(-ER1/10); % Note)Extinction Ratio [dB] = -10log10(1-ModulationIndex)
mod_out1 = ModulatorAmplitude(Laser1,E_elec,ModulationIndex1);

ER2 = 30;% Extinction Ratio [dB]
ModulationIndex2 = 1-10^(-ER2/10); % Note)Extinction Ratio [dB] = -10log10(1-ModulationIndex)
mod_out2 = ModulatorAmplitude(Laser1,E_elec,ModulationIndex2);

%-----------------------------------------------------
figure(1)
plot(t_range,abs(mod_out1).^2,t_range,abs(mod_out2).^2,'r' )
title('ұ ȭ')
xlabel('[sec]'), ylabel('[W]')
legend('ұ=10dB','ұ=30dB')
%-----------------------------------------------------

ERiseTime_v = [1/3/BitRate 1/8/BitRate 1/16/BitRate]; %

for k=1:length(ERiseTime_v)
    
    E_elec_v = RiseTimeAdjust(E_data,ERiseTime_v(k));
    mod_out(k,:) = ModulatorAmplitude(Laser1,E_elec_v,ModulationIndex2);
end

figure(2)
plot(t_range,abs(mod_out(1,:)).^2,t_range,abs(mod_out(2,:)).^2,'g',t_range,abs(mod_out(3,:)).^2,'r' )
title('½ð ȭ')
xlabel('[sec]'), ylabel('[W]')
legend('½ð=1/3/B', '½ð=1/8/B','½ð=1/16/B')
%--------------------------------------------------------------------

RZRatio = 0.3:0.2:0.7;
ERiseTime = 1/8/BitRate; %  In the form of k/BitRate, k should be less than (1/2)*RZRatio if RZCoder used   

for k=1:length(RZRatio)
    
    E_dataRZ = RZCoder(info_bits,RZRatio(k));
    E_elecRZ = RiseTimeAdjust(E_dataRZ,ERiseTime);
    mod_outRZ(k,:) = ModulatorAmplitude(Laser1,E_elecRZ,ModulationIndex2);
end

figure(3)
plot(t_range,abs(mod_outRZ(1,:)).^2,t_range,abs(mod_outRZ(2,:)).^2,'g',t_range,abs(mod_outRZ(3,:)).^2,'r' )
title('RZRatio ȭ')
xlabel('[sec]'), ylabel('[W]')
legend('RZRatio=0.3', 'RZRatio=0.5','RZRatio=0.7')
