% Fig8-15 (ShotNoiseFactor = 1)
% Fig8-16 (ShotNoiseFactor = 0)

clear all ;% clear all variables
close all

%============== Define global variables ===================================================
global BitRate Bits SampleRate DataLength samplesperbit t_range f_sym

BitRate = 1e9;  
Bits2Power = 6;
Bits = 2^Bits2Power;
TimeWindow = Bits/BitRate;
SampleRate = BitRate*2^10;
DataLength = TimeWindow*SampleRate;
Ref_Frequency = 193.1e12; %[Hz] Reference optical frequency
samplesperbit = SampleRate/BitRate; % data number per bit

% time, optical frequency range
t_range = 0:1/SampleRate:1/SampleRate*(DataLength-1);
f_sym = SampleRate*(-DataLength/2:(DataLength/2-1))/DataLength; %

%==========================================================================================
%=================== Define Simulation Blocks =============================================
%------------- Transmitter -------------------%
% SM Laser
UnmodulatedPower = 1e-3; %[W] 
Linewidth = 10e6; %[Hz]
Laser1 = LaserCW(UnmodulatedPower,Linewidth);

PreSpaces = 2;  %number of leading zero bits
PostSpaces = 2; %number of trailing zero bits
info_bits = infobits_PRBS(PreSpaces,PostSpaces);

E_data = NRZCoder(info_bits);

ERiseTime = 1/4/BitRate; %  In the form of k/BitRate, k should be less than (1/2)*RZRatio if RZCoder used   
E_elec = RiseTimeAdjust(E_data,ERiseTime);

ModulationIndex = 0.9;
mod_out = ModulatorAmplitude(Laser1,E_elec,ModulationIndex);

%---------------- Transmission -------------------------------------
loss = 20; % [dB]

mod_att = Attenuator(mod_out,loss);

%--------------- Receiver1 (PIN) ------------------------------------------
Responsivity = 1.0;% [A/W], [0,2]
ThermalNoiseDen = 1e-22;% [A^2/Hz] 
DarkCurrent = 0;% [A], [0,10e-6]
ShotNoiseFactor = 0;%   (1 if shot noise included, 0 if not)

y_PIN = PhotoDiode_PIN(mod_att,Responsivity,ThermalNoiseDen,DarkCurrent,ShotNoiseFactor); 

FilterOrder = 3;
BW3dB = 0.75*BitRate;

y_PIN_LP = E_FilterLPBessel(y_PIN,FilterOrder,BW3dB);

%--------------- Receiver2 (APD) ------------------------------------------
DarkCurrMult = 0 ;%[A], [0,10e-6]
DarkCurrNonMult = 0 ;% [A], [0.10e-6]
AvalacheMul = 20; %   [1,1000]
IonizationCoeff = 0.6;%  [1e-3, 1]

y_APD = PhotoDiode_APD(mod_att,Responsivity,AvalacheMul,IonizationCoeff,...
    ShotNoiseFactor,ThermalNoiseDen,DarkCurrMult,DarkCurrNonMult);

y_APD_LP = E_FilterLPBessel(y_APD,FilterOrder,BW3dB);


figure(1)
subplot 211
plot(t_range,y_PIN)
subplot 212
plot(t_range,y_PIN_LP)

figure(2)
subplot 211
plot(t_range,y_APD)
subplot 212
plot(t_range,y_APD_LP)


Res_Filter_Type = 'Rectangular';
Res_Filter_BW = BitRate/(Bits/4);
Pf = RFSpectrumAnalyzer(y_APD_LP,Res_Filter_BW,Res_Filter_Type)

[Pe, Ve] = PowerMeterElec(y_APD_LP);

% Power calculated by 'RFSpectrumAnalyzer' and  by 'PowerMeterElec' should be equal to each other.

